using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading;
using Dapfor.Net.Data;
using Dapfor.Net.Grid.Tutorial.BusinessData.CAC40;
using Dapfor.Net.Grid.Tutorial.BusinessData.Currencies;
using Dapfor.Net.Grid.Tutorial.BusinessData.OrderBook;

namespace Dapfor.Net.Grid.Tutorial.BusinessData
{
    public class Provider
    {
        private static Provider _instance;
        private readonly IList<string> _availableCurrencies = new List<string>();

        private readonly IList<Feeder> _cac40Feeders = new ThreadSafeBindingList<Feeder>();
        private readonly IDictionary<string, CurrencyRate> _currencyRates = new Dictionary<string, CurrencyRate>();
        private readonly IList<Instrument> _instruments = new List<Instrument>();
        private readonly IList<Market> _markets = new ThreadSafeBindingList<Market>();
        private readonly ThreadSafeBindingList<Order> _singleOrders = new ThreadSafeBindingList<Order>();
        private readonly ThreadSafeBindingList<Strategy> _strategies = new ThreadSafeBindingList<Strategy>();

        private readonly Random _random = new Random();
        private Timer _timer;

        //Create fake data
        public Provider()
        {
            Market market = new Market("NYSE Euronext", new DateTime(DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day, 9, 0, 0), new DateTime(DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day, 17, 30, 0), true);
            _markets.Add(market);


            //Create feeders
            _cac40Feeders.Add(new Feeder(new Instrument("ACCOR", "FR0000120404", "AC", 227556372, 5292961213, 0.50, "EUR"), market, 22.96, 546, 137252, 23.22, 23.20, 23.305, 22.90));
            _cac40Feeders.Add(new Feeder(new Instrument("AIR LIQUIDE", "FR0000120073", "AI", 282931628, 23975626157, 3.74, "EUR"), market, 84.99, 54, 118085, 84.12, 84.16, 85.20, 84.16));
            _cac40Feeders.Add(new Feeder(new Instrument("ALCATEL-LUCENT", "FR0000130007", "ALU", 2318112376, 4870354102, 0.73, "EUR"), market, 2.087, 1032, 2229578, 2.084, 2.088, 2.096, 2.068));
            _cac40Feeders.Add(new Feeder(new Instrument("ALSTOM", "FR0010220475", "ALO", 294113845, 10754272742, 1.21, "EUR"), market, 36.135, 1000, 461970, 36.432, 36.425, 36.52, 36.00));
            _cac40Feeders.Add(new Feeder(new Instrument("ARCELORMITTAL", "LU0323134006", "MT", 1560914610, 35463979939, 3.03, "EUR"), market, 22.81, 3645, 1825581, 22.483, 22.485, 22.94, 22.455));
            _cac40Feeders.Add(new Feeder(new Instrument("AXA", "FR0000120628", "CS", 2289996372, 29506603253, 3.52, "EUR"), market, 12.695, 6532, 1494925, 12.704, 12.705, 12.88, 12.65));
            _cac40Feeders.Add(new Feeder(new Instrument("BNP PARIBAS ACT.A", "FR0000131104", "BNP", 1193842336, 56182220332, 7.57, "EUR"), market, 46.835, 650, 663646, 46.61, 46.62, 46.98, 46.27));
            _cac40Feeders.Add(new Feeder(new Instrument("BOUYGUES", "FR0000120503", "EN", 355604191, 10997059607, 1.30, "EUR"), market, 30.785, 65, 214496, 30.71, 30.77, 31.00, 30.47));
            _cac40Feeders.Add(new Feeder(new Instrument("CAP GEMINI", "FR0000125338", "CAP", 155031166, 5618329456, 0.83, "EUR"), market, 36.275, 694, 180459, 36.197, 36.195, 36.33, 36.06));
            _cac40Feeders.Add(new Feeder(new Instrument("CARREFOUR", "FR0000120172", "CA", 704902716, 24527090003, 3.24, "EUR"), market, 34.86, 368, 541224, 34.540, 34.535, 35.11, 34.50));
            _cac40Feeders.Add(new Feeder(new Instrument("CREDIT AGRICOLE", "FR0000045072", "ACA", 647921840, 21316463419, 1.32, "EUR"), market, 8.872, 3540, 1115162, 8.870, 8.869, 8.909, 8.694));
            _cac40Feeders.Add(new Feeder(new Instrument("DANONE", "FR0000120644", "BN", 2385993219, 29218035375, 4.09, "EUR"), market, 45.365, 93, 338294, 44.78, 44.79, 45.44, 44.785));
            _cac40Feeders.Add(new Feeder(new Instrument("DEXIA", "BE0003796134", "DEXB", 1846406344, 5792176701, 0.46, "EUR"), market, 3.11, 1360, 642095, 3.155, 3.154, 3.19, 3.096));
            _cac40Feeders.Add(new Feeder(new Instrument("EADS", "NL0000235190", "EAD", 816150511, 13401191391, 1.04, "EUR"), market, 16.515, 1035, 518458, 16.31, 16.30, 16.64, 16.30));
            _cac40Feeders.Add(new Feeder(new Instrument("EDF", "FR0010242511", "EDF", 1848866662, 55992926859, 1.34, "EUR"), market, 30.445, 650, 241071, 30.254, 30.155, 30.60, 30.155));
            _cac40Feeders.Add(new Feeder(new Instrument("ESSILOR INTL.", "FR0000121667", "EI", 211853120, 10507914752, 1.57, "EUR"), market, 49.705, 469, 85727, 49.38, 49.37, 49.75, 49.315));
            _cac40Feeders.Add(new Feeder(new Instrument("FRANCE TELECOM", "FR0000133308", "FTE", 2648709774, 39015494971, 4.27, "EUR"), market, 14.85, 65, 1793751, 14.64, 14.65, 14.875, 14.65));
            _cac40Feeders.Add(new Feeder(new Instrument("GDF SUEZ", "FR0010208488", "GSZ", 2261371335, 54996550867, 4.83, "EUR"), market, 24.66, 87, 1456380, 24.20, 24.19, 24.75, 24.19));
            _cac40Feeders.Add(new Feeder(new Instrument("L'OREAL", "FR0000120321", "OR", 599217810, 49447453681, 3.08, "EUR"), market, 83.04, 981, 224852, 82.51, 82.50, 83.20, 81.41));
            _cac40Feeders.Add(new Feeder(new Instrument("LAFARGE", "FR0000120537", "LG", 286453317, 11433784148, 1.27, "EUR"), market, 40.355, 64, 260713, 39.830, 39.825, 40.43, 39.67));
            _cac40Feeders.Add(new Feeder(new Instrument("LAGARDERE S.C.A.", "FR0000130213", "MMB", 131133286, 3381927446, 0.42, "EUR"), market, 25.735, 94, 38.739, 25.560, 25.595, 25.97, 25.59));
            _cac40Feeders.Add(new Feeder(new Instrument("LVMH", "FR0000121014", "MC", 489982207, 43843607882, 3.41, "EUR"), market, 89.03, 84, 153793, 88.91, 88.92, 89.48, 88.53));
            _cac40Feeders.Add(new Feeder(new Instrument("MICHELIN", "FR0000121261", "ML", 149172216, 8783260078, 1.08, "EUR"), market, 58.30, 14, 159521, 58.42, 58.43, 58.77, 57.65));
            _cac40Feeders.Add(new Feeder(new Instrument("PERNOD RICARD", "FR0000120693", "RI", 263881847, 16819828928, 2.11, "EUR"), market, 63.33, 5, 171828, 62.98, 62.99, 64.14, 62.98));
            _cac40Feeders.Add(new Feeder(new Instrument("PEUGEOT", "FR0000121501", "UG", 234048798, 5564510172, 0.55, "EUR"), market, 23.435, 47, 339451, 23.55, 23.56, 23.74, 23.205));
            _cac40Feeders.Add(new Feeder(new Instrument("PPR", "FR0000121485", "PP", 126579388, 12551612114, 1.17, "EUR"), market, 98.72, 36, 60088, 98.51, 98.50, 98.92, 97.45));
            _cac40Feeders.Add(new Feeder(new Instrument("RENAULT", "FR0000131906", "RNO", 295722284, 10030899873, 1.01, "EUR"), market, 33.42, 41, 335215, 33.71, 33.59, 33.835, 33.07));
            _cac40Feeders.Add(new Feeder(new Instrument("SAINT GOBAIN", "FR0000125007", "SGO", 530800865, 16624683092, 2.05, "EUR"), market, 31.455, 32, 415829, 31.02, 31.01, 31.53, 31.005));
            _cac40Feeders.Add(new Feeder(new Instrument("SANOFI-AVENTIS", "FR0000120578", "SAN", 1310823566, 62486959391, 7.83, "EUR"), market, 48.25, 12, 681221, 47.25, 47.26, 48.48, 47.225));
            _cac40Feeders.Add(new Feeder(new Instrument("SCHNEIDER ELECTRIC", "FR0000121972", "SU", 270090217, 22954967543, 3.02, "EUR"), market, 84.94, 43, 256086, 85.02, 85.01, 85.45, 84.12));
            _cac40Feeders.Add(new Feeder(new Instrument("SOCIETE GENERALE", "FR0000130809", "GLE", 742130152, 26939324518, 3.65, "EUR"), market, 35.97, 73, 1254372, 35.790, 35.795, 36.165, 35.305));
            _cac40Feeders.Add(new Feeder(new Instrument("STMICROELECTRONICS", "NL0000226223", "STM", 910319305, 6098229024, 0.66, "EUR"), market, 6.649, 184, 852413, 6.620, 6.621, 6.685, 6.606));
            _cac40Feeders.Add(new Feeder(new Instrument("SUEZ ENVIRONNEMENT", "FR0010613471", "SEV", 489699060, 6809265429, 0.58, "EUR"), market, 13.82, 73, 141465, 13.930, 13.935, 13.945, 13.795));
            _cac40Feeders.Add(new Feeder(new Instrument("TECHNIP", "FR0000131708", "TEC", 109343295, 5255585474, 0.73, "EUR"), market, 47.72, 6, 101199, 47.610, 47.615, 48.065, 47.135));
            _cac40Feeders.Add(new Feeder(new Instrument("TOTAL", "FR0000120271", "FP", 2348422885, 89345748660, 12.56, "EUR"), market, 38.09, 87, 1328042, 37.91, 37.905, 3830, 37.75));
            _cac40Feeders.Add(new Feeder(new Instrument("UNIBAIL-RODAMCO", "FR0000124711", "UL", 91444032, 12335799917, 1.93, "EUR"), market, 135.55, 13, 73249, 134.14, 134.15, 136.40, 133.70));
            _cac40Feeders.Add(new Feeder(new Instrument("VALLOUREC", "FR0000120354", "VK", 116548468, 8938102011, 1.29, "EUR"), market, 75.91, 64, 331394, 75.81, 75.82, 76.37, 75.01));
            _cac40Feeders.Add(new Feeder(new Instrument("VEOLIA ENVIRON.", "FR0000124141", "VIE", 497362392, 9728408388, 1.30, "EUR"), market, 19.42, 9, 401104, 19.41, 19.42, 19.565, 19.33));
            _cac40Feeders.Add(new Feeder(new Instrument("VINCI", "FR0000125486", "DG", 549682659, 19241641478, 2.57, "EUR"), market, 35.005, 16, 438.552, 34.69, 34.70, 35.14, 34.635));
            _cac40Feeders.Add(new Feeder(new Instrument("VIVENDI", "FR0000127771", "VIV", 1229346769, 21267699104, 3.15, "EUR"), market, 17.285, 21, 855083, 17.150, 17.155, 17.37, 17.15));

            //Populate instrument collection
            foreach (Feeder feeder in _cac40Feeders)
            {
                _instruments.Add(feeder.Instrument);
            }

            //Create collection of available currencies
            _availableCurrencies.Add("USD");
            _availableCurrencies.Add("EUR");
            _availableCurrencies.Add("CAD");
            _availableCurrencies.Add("GBP");
            _availableCurrencies.Add("JPY");
            _availableCurrencies.Add("AUD");
            _availableCurrencies.Add("ZAR");
            _availableCurrencies.Add("SEK");
            _availableCurrencies.Add("CHF");
            _availableCurrencies.Add("DKK");
            _availableCurrencies.Add("HKD");
            _availableCurrencies.Add("INR");
            _availableCurrencies.Add("ILS");
            _availableCurrencies.Add("ISK");
            _availableCurrencies.Add("MAD");
            _availableCurrencies.Add("RUB");
            _availableCurrencies = new ReadOnlyCollection<string>(_availableCurrencies);

            foreach (string currency in _availableCurrencies)
            {
                _currencyRates.Add(currency, new CurrencyRate(currency, _availableCurrencies));
            }

            //Initialize currency rates with some values:
            CurrencyRates["USD"]["CAD"].Rate = 1/0.96367;
            CurrencyRates["USD"]["EUR"].Rate = 1/1.2852;
            CurrencyRates["USD"]["GBP"].Rate = 1/1.5269;
            CurrencyRates["USD"]["JPY"].Rate = 1/0.011496;
            CurrencyRates["USD"]["AUD"].Rate = 1/0.8838;
            CurrencyRates["USD"]["ZAR"].Rate = 1/0.132371;
            CurrencyRates["USD"]["SEK"].Rate = 1/0.135685;
            CurrencyRates["USD"]["CHF"].Rate = 1/0.951022;
            CurrencyRates["USD"]["DKK"].Rate = 1/0.172467;
            CurrencyRates["USD"]["HKD"].Rate = 1/0.128602;
            CurrencyRates["USD"]["INR"].Rate = 1/0.021157;
            CurrencyRates["USD"]["ILS"].Rate = 1/0.259168;
            CurrencyRates["USD"]["ISK"].Rate = 1/0.006138;
            CurrencyRates["USD"]["RUB"].Rate = 1/0.032861;
            CurrencyRates["USD"]["MAD"].Rate = 1/0.115765;

            //Create cross rates
            foreach (KeyValuePair<string, CurrencyRate> rate in CurrencyRates)
            {
                string currency1 = rate.Key;
                if (currency1 != "USD")
                {
                    foreach (string currency2 in AvailableCurrencies)
                    {
                        double usdRate1 = CurrencyRates["USD"][currency1].Rate;
                        double usdRate2 = CurrencyRates["USD"][currency2].Rate;
                        if (usdRate1 > 0 && usdRate2 > 0)
                        {
                            rate.Value[currency2].Rate = usdRate2/usdRate1;
                        }
                    }
                }
            }


            //Populate with random instruments
            Random r = new Random();
            for (int i = 0; i < 100000; ++i)
            {
                string isin = string.Format("{0}{1}{2:0000000000}", (char) ('A' + r.Next(24)), (char) ('A' + r.Next(24)), r.Next(100000, 10000000) + i);

                _instruments.Add(new Instrument("instriment " + i, isin, "MNEMO" + i, 0, 0, 0, AvailableCurrencies[r.Next(AvailableCurrencies.Count)]));
            }
        }

        #region Public properties

        public IList<Feeder> Cac40Feeders
        {
            get { return _cac40Feeders; }
        }

        public IList<Instrument> Instruments
        {
            get { return _instruments; }
        }

        public IList<Market> Markets
        {
            get { return _markets; }
        }

        public IList<string> AvailableCurrencies
        {
            get { return _availableCurrencies; }
        }

        public IDictionary<string, CurrencyRate> CurrencyRates
        {
            get { return _currencyRates; }
        }

        public IList<Order> SingleOrders
        {
            get { return _singleOrders; }
        }

        public IList<Strategy> Strategies
        {
            get { return _strategies; }
        }

        #endregion

        public static Provider Instance
        {
            get { return _instance ?? (_instance = new Provider()); }
        }

        #region Simulation of real-time updates 

        public void BeginUpdates()
        {
            if (_timer == null)
            {
                _timer = new Timer(OnTick, null, 10, 10);
            }
        }

        private void OnTick(object state)
        {
            Feeder feeder = Cac40Feeders[_random.Next(Cac40Feeders.Count)];
            switch (_random.Next()%8)
            {
                case 0: //Updates bid values
                    feeder.Bid = feeder.Last - feeder.Last*(1f/_random.Next(500, 1000));
                    break;

                case 1: //Updates bid quantity
                    feeder.BidQty = _random.Next(1, 50);
                    break;


                case 2: //Updates ask values
                    feeder.Ask = feeder.Last + feeder.Last*(1f/_random.Next(500, 1000));
                    break;

                case 3: //Updates ask quantity
                    feeder.AskQty = _random.Next(1, 50);
                    break;

                case 4: //Update the 'last'
                    double delta = feeder.Last*(1f/_random.Next(500, 1000));
                    delta = _random.Next()%2 == 0 ? delta : -delta;

                    feeder.Last = Math.Max(feeder.Last + delta, feeder.Open/2);
                    break;

                case 5: //Update the 'last'
                    feeder.LastQty = _random.Next(1, 50);
                    break;

                case 6: //Update the market state
                    if (_random.Next()%10 == 0)
                    {
                        int v = _random.Next(100);
                        if (v > 5)
                        {
                            feeder.State = FeederState.Ok;
                        }
                        else feeder.State = v > 2 ? FeederState.PartialOk : FeederState.Nok;
                    }
                    break;

                case 7: //Update currency rate
                    string currency1 = AvailableCurrencies[_random.Next(AvailableCurrencies.Count)];
                    string currency2 = AvailableCurrencies[_random.Next(AvailableCurrencies.Count)];
                    if (!Equals(currency1, currency2))
                    {
                        double rate = CurrencyRates[currency1][currency2].Rate;
                        double coeff = 1/((double) (_random.Next(200, 400)));
                        coeff = _random.Next()%2 == 0 ? coeff : -coeff;
                        rate *= (1 + coeff);
                        rate = rate == 0 ? CurrencyRates[currency1][currency2].Rate : rate;
                        CurrencyRates[currency1][currency2].Rate = rate;
                        CurrencyRates[currency2][currency1].Rate = 1/rate;
                    }
                    break;
            }
        }

        public void StopUpdates()
        {
            if (_timer != null)
            {
                _timer.Dispose();
            }
        }

        #endregion
    }
}