using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Editors;
using Dapfor.Net.Grid.Tutorial.BusinessData.OrderBook;
using Dapfor.Net.Ui;
using Appearance = Dapfor.Net.Ui.Appearance;

namespace Dapfor.Net.Grid.Tutorial.Editors
{
    public class StrategyStatusEditor : UITypeEditorEx
    {
        private readonly Font _font;

        public StrategyStatusEditor()
        {
            try
            {
                _font = new Font("Microsoft Sans Serif", 8.25f, FontStyle.Bold);
            }
            catch
            {
                _font = SystemFonts.MenuFont;
            }
        }


        public override bool GetPaintCellSupported()
        {
            return true;
        }

        public override void PaintCell(PaintCellEventArgs e)
        {
            Strategy strategy = e.Cell.Row.DataObject as Strategy;
            e.Parts ^= e.Parts & PaintPart.Text;

            if (strategy != null)
            {
                e.PaintAll();
                e.Handled = true;

                Rectangle bounds = e.Cell.VirtualBounds;
                bounds.Inflate(-2, -1);

                int activeOrders = 0;
                foreach (Order order in strategy.Orders)
                {
                    if (Equals(Status.Active, order.Status))
                    {
                        activeOrders++;
                    }
                }

                //Draw inactive button
                double width = bounds.Width;
                width *= (double) activeOrders/strategy.Orders.Count;
                Rectangle inactiveRc = bounds;
                inactiveRc.Width = (int) width;
                e.Render.DrawCaption(new Appearance(SystemColors.WindowText, Color.LightGreen), ElementState.Normal, inactiveRc, BorderSide.All, e.Graphics);

                //Draw active button
                Rectangle activeRc = bounds;
                activeRc.Width = bounds.Width - inactiveRc.Width;
                activeRc.X = inactiveRc.Right;
                e.Render.DrawCaption(new Appearance(SystemColors.WindowText, ControlPaint.LightLight(Color.OrangeRed)), ElementState.Normal, activeRc, BorderSide.All, e.Graphics);

                if (strategy.Orders.Count > 0)
                {
                    string text = "Start all";
                    switch (strategy.Status)
                    {
                        case Status.Active:
                            text = "Stop all";
                            break;
                    }

                    e.Font = _font;
                    e.PaintText(text, bounds);
                }
            }
        }

        public override StopEditReason EditCell(IGridEditorService service, Cell cell, StartEditReason reason)
        {
            Strategy strategy = cell.Row.DataObject as Strategy;
            if (strategy != null)
            {
                int activeOrders = 0;
                foreach (Order order in strategy.Orders)
                {
                    if (Equals(Status.Active, order.Status))
                    {
                        activeOrders++;
                    }
                }

                //Deactivate or activate all orders
                foreach (Order order in strategy.Orders)
                {
                    order.Status = activeOrders > 0 ? Status.Cancelled : Status.Active;
                }
            }
            return StopEditReason.UserStop;
        }
    }
}